// -- Ray Pong
// games.neoclaw.net/raypong

//main game loop
//Luke Lenhart, 2008
//See the MPMA Framework license for details on how this code may be used.

#include <iostream>
#include <stdlib.h>
#include <time.h>

#include "mpma/base/Setup.h"
#include "mpma/base/Memory.h"
#include "mpma/base/Thread.h"
#include "mpma/base/DebugRouter.h"
#include "mpma/base/Profiler.h"
#include "mpma/gfxsetup/Setup.h"

#include "Game.h"
#include "Render.h"
#include "NetStuff.h"

void MemoryErrorReport(const std::string &msg)
{
    std::cout<<msg;
    std::cout.flush();
}

int main()
{
    srand((uint)time(0));

    //init framework
    std::cout<<"Setting up..."<<std::endl;
    {
        MPMA::InitAndShutdown frameworkInit;

        mpmaMemoryManager.AddLeakReportCallback(MemoryErrorReport);
        MPMA::RouterOutputStdout *stdReportOutput=new3(MPMA::RouterOutputStdout());
        MPMA::ErrorReport().AddOutputMethod(stdReportOutput);

        //ask user for mode
        int gameMode;
        bool setupGood=true;
        do
        {
            std::cout<<"\nGame mode:\n";
            std::cout<<" 1. Single player.\n";
            std::cout<<" 2. Multiplayer."<<std::endl;
            std::string input;
            std::cin>>input;
            gameMode=MPMA::Vary(input);
        } while (!(gameMode==1 || gameMode==2));

        if (gameMode==2)
        {
            if (!PongNet::NegotiateConnection())
            {
                std::cout<<"Failed to set up network game."<<std::endl;
                setupGood=false;
            }
        }

        if (setupGood)
        {
            //create the window
            GFX::GraphicsSetup gfx;
            gfx.Name="Ray Pong";
#ifdef _DEBUG
            gfx.Width=100;
            gfx.Height=100;
#else
            gfx.Width=400;
            gfx.Height=400;
#endif
            gfx.Minimized=true; //start minimized
            SetupWindow(gfx);
            gfx.Minimized=false; //pull out, so the windows comes up in front of the console.. odd hack related to using std::cin..
            SetupWindow(gfx);

            std::cout<<"Starting game in 1s!"<<std::endl;
            MPMA::Timer gameTimer;

            //main loop
            std::cout<<"Running..."<<std::endl;
            for ( ;GFX::GetWindowState() ;)
            {
                ProfileScope("Frame Update");

                if (gameTimer.Step(false)>1) //delay start by 1s
                    Game::Update();
                Renderer::DrawScene();
                GFX::UpdateWindow();

                if (PongNet::IsNetGame)
                {
                    if (!PongNet::Update())
                    {
                        std::cout<<"Lost connection to other player."<<std::endl;
                        break;
                    }
                }

                MPMA::Sleep(0); //prevent stalling the entire system
            }
        }

        //print final score
        std::cout<<"Final score:\n"<<
            " -"<<Game::PLeftName<<": "<<Game::LeftScore<<"\n"<<
            " -"<<Game::PRightName<<": "<<Game::RightScore<<std::endl;

        //clean up
        std::cout<<"Shutting down..."<<std::endl;
        Renderer::Cleanup();
        GFX::ShutdownWindow();
        PongNet::Cleanup();
        delete3(stdReportOutput);
    }

    //pause a second, so silly people running from a gui that may miss any output have a chance to see it in the console
    std::cout<<"\nDone."<<std::endl;
    MPMA::Sleep(1000);

    return 0;
}

