//!\file Types.h \brief Definition of basic data types, so there is no ambiguity between different platforms and compilers.
//!\details
//!The following types are defined:       \n
//! uint8  - 8 bit unsigned integer       \n
//! sint8  - 8 bit signed integer         \n
//! uint16 - 16 bit unsigned integer      \n
//! sint16 - 16 bit signed integer        \n
//! uint32 - 32 bit unsigned integer      \n
//! sint32 - 32 bit signed integer        \n
//! uint64 - 64 bit unsigned integer      \n
//! sint64 - 64 bit signed integer        \n
//! uint   - unsigned integer of the size native to the cpu architecture   \n
//! sint   - signed integer of the size native to the cpu architecture     \n
//!
//! POINTER_SIZE - number of bytes needed to store a pointer
//!
//!The following data types can also be assumed for all platforms: \n
//! char   - 8 bit signed int (sint8)           \n
//! float  - 32 bit floating point number       \n
//! double - 64 bit floating point number       \n
//!
//!The following storage specifiers are defined: \n
//! THREAD_LOCAL  - variable is allocated from thread-local storage

//Luke Lenhart, 2007
//See /docs/License.txt for details on how this code may be used.

// -- platform implementations --

#if defined(WIN32) // msvc - 32 bit
    typedef unsigned int      uint;
    typedef signed int        sint;
    typedef unsigned char     uint8;
    typedef signed char       sint8;
    typedef unsigned short    uint16;
    typedef signed short      sint16;
    typedef unsigned int      uint32;
    typedef signed int        sint32;
    typedef unsigned __int64  uint64;
    typedef signed __int64    sint64;
    #define POINTER_SIZE      sizeof(void*)
    #define THREAD_LOCAL      __declspec(thread)
#elif defined(linux)  // g++ - 32 bit
    typedef unsigned int      uint;
    typedef signed int        sint;
    typedef unsigned char     uint8;
    typedef signed char       sint8;
    typedef unsigned short    uint16;
    typedef signed short      sint16;
    typedef unsigned int      uint32;
    typedef signed int        sint32;
    typedef unsigned long long uint64;
    typedef signed long long  sint64;
    #define POINTER_SIZE      sizeof(void*)
    #define THREAD_LOCAL      __thread
#else
    #error Unknown platform in types.h
#endif
